import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { BaseDbService } from './../commons/services/base-db.service';
import { Injectable } from '@nestjs/common';
import { CreateTagInput } from './dto/create-tag.input';
import { UpdateTagInput } from './dto/update-tag.input';
import { Tag } from './entities/tag.entity';

@Injectable()
export class TagsService extends BaseDbService<Tag> {
  readonly uniqueFields: Array<keyof Tag> = ['name'];
  constructor(@InjectRepository(Tag) readonly repository: Repository<Tag>) {
    super(repository);
  }

  /**
   * create or recover a tag
   */
  async create(createTagInput: CreateTagInput): Promise<Tag> {
    const old = await this.repository.findOne({ name: createTagInput.name });
    return this.repository.save(
      old
        ? this.repository.merge(old, createTagInput)
        : this.repository.create(createTagInput),
    );
  }

  async findAll() {
    return await this.repository.find({
      order: { createdAt: 'DESC' },
    });
  }

  async update(tag: Tag, updateTagInput: UpdateTagInput) {
    await this.isDuplicateEntityForUpdate(tag.id, updateTagInput);
    return await this.repository.save(
      this.repository.merge(tag, updateTagInput),
    );
  }

  async remove(id: string) {
    await this.canRemove([id]);
    return await this.repository.softDelete({ id }).then((d) => d.affected);
  }
}
